#!/bin/sh
# This file is part of the Linux NIC driver for Emulex OneConnect CNA.
#
# Copyright (C) 2005-2012 Emulex. All rights reserved.
#
# EMULEX and SLI are trademarks of Emulex.
# www.emulex.com
# linux-drivers@emulex.com
#Version:1.1
#Date:05-FEB-2013
#Modified: A.Madhusudana Reddy
# This program is free software; you can redistribute it and/or modify
# it under the terms of version 2 of the GNU General Public License as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful.
# ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
# INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE DISCLAIMED, EXCEPT TO THE
# EXTENT THAT SUCH DISCLAIMERS ARE HELD TO BE LEGALLY INVALID.
# See the GNU General Public License for more details, a copy of which
# can be found in the file COPYING included with this package
#
# This file collects info about the system on which the be2net driver is loaded.
# All collected data are bundled into the file `hostname`.tar.gz
#

#Either provide the duration (in seconds) or default is 30 seconds.
if [ $# -ne 1 ]; then
	DURATION=30
else
	DURATION=$1
fi

DELAY=5
COUNT=`expr $DURATION / $DELAY`
echo `date` "Gathering data. This takes about $DURATION seconds. Please wait.."

decor ()
{
	echo "" >> $1
	echo "$2" >> $1
	echo "----------------------------------" >> $1
}

echo `date` > sys_info.txt
echo "system report on which be2net is loaded:" >> sys_info.txt
echo "Sample duration:" $DURATION >> sys_info.txt
i=0
k=0
for k in `ls /sys/class/net/ |  awk -F " " '{print $1}'`
do
	if [[ "$k" != "lo" && "$k" != "sit0" && "$k" != "virbr0" ]] ; then
        	module=`ethtool -i "$k" | grep driver | cut -d ":" -f2 | sed -e "s/^ *//"`
                if [ "$module" = "be2net" ];then
                	intrf[$i]=$k
                        ((i++))
                fi
        fi

done

decor sys_info.txt "id -a"
id -a >> sys_info.txt

decor sys_info.txt "uname -a"
uname -a >> sys_info.txt

if [ `uname -i` == "x86_64" -a -x ./cpu_topology64 ]
then
	decor sys_info.txt "CPU topoloy"
	./cpu_topology64 >> sys_info.txt
elif test -x ./cpu_topology32 ; then
	decor sys_info.txt "CPU topoloy"
	./cpu_topology32 >> sys_info.txt
else
	decor sys_info.txt "CPU topology not available"
fi

if test -f /usr/bin/lscpu; then
	decor sys_info.txt "lscpu"
	lscpu >> sys_info.txt
fi

for f in /etc/cat /etc/redhat-release /etc/SuSE-release;
do
	if test -r $f ; then
		decor sys_info.txt "$f"
		cat "$f" >> sys_info.txt
	fi
done

for f in /proc/cpuinfo /proc/meminfo /proc/version /proc/uptime  \
	/proc/cmdline /proc/interrupts /proc/net/dev /proc/iomem;
do
	decor sys_info.txt "$f"
	cat $f >> sys_info.txt
done

decor sys_dmesg.txt dmesg
dmesg >> sys_dmesg.txt

decor sys_lspci_info.txt "lspci -vvv -xxxx -d 19a2:*"
lspci -vvv -xxxx -d 19a2:* >> sys_lspci_info.txt

decor sys_info.txt lsmod
lsmod >> sys_info.txt

decor sys_info.txt "dmi info"
dmidecode >> sys_info.txt

if test -d /sys/module/be2net/parameters; then 
	decor be2net_info.txt "be2net module parameters"
	find /sys/module/be2net/parameters -type f >> be2net_info.txt
	find /sys/module/be2net/parameters -type f | xargs head >> be2net_info.txt
fi

if test -f /proc/config.gz ; then
	decor sys_info.txt "/proc/config.gz"
	zcat /proc/config.gz >> sys_info.txt
else
	if test -f /boot/config-`uname -r` ; then
		decor sys_info.txt "if test -f /boot/config-`uname -r`"
		cat /boot/config-`uname -r` >> sys_info.txt
	else
		decor sys_info.txt "/lib/modules/`uname -r`/build/.config"
		cat "/lib/modules/`uname -r`/build/.config" >> sys_info.txt
	fi
fi

decor sys_info.txt "sysctl info"
sysctl -a >> sys_info.txt

for intf in ${intrf[@]}
do
		decor be2net_info.txt "Numa Node for $intf"
		bdf=`ethtool -i $intf | grep bus | cut -b11-`
		if test -f /sys/bus/pci/devices/$bdf/numa_node; then
			cat /sys/bus/pci/devices/$bdf/numa_node >> be2net_info.txt
		fi
		decor be2net_info.txt "Local CPUs for $intf"
		if test -f /sys/bus/pci/devices/$bdf/local_cpus; then
			cat /sys/bus/pci/devices/$bdf/local_cpus >> be2net_info.txt
		fi
		if test -f /sys/bus/pci/devices/$bdf/local_cpulist; then
			cat /sys/bus/pci/devices/$bdf/local_cpulist >> be2net_info.txt
		fi
		decor be2net_info.txt "----------------"
		decor be2net_info.txt "ethtool $intf"
		ethtool $intf >> be2net_info.txt
		decor be2net_info.txt "ethtool -i $intf"
		ethtool -i $intf >> be2net_info.txt
		decor be2net_info.txt "ethtool -k $intf"
		ethtool -k $intf >> be2net_info.txt 2>&1
		decor be2net_info.txt "ethtool -c $intf"
		ethtool -c $intf >> be2net_info.txt
		decor be2net_info.txt "ethtool -a $intf"
		ethtool -a $intf >> be2net_info.txt
done

for f in /var/log/dmesg /var/log/boot.msg;
do
	if test -r $f ; then
		decor sys_info.txt $f
		cat $f >> sys_info.txt
	fi
done

if test -d /etc/sysconfig/network-scripts; then
	rh=1
	decor sys_info.txt "contents of /etc/sysconfig/network-scripts"
	ls -l /etc/sysconfig/network-scripts/ >> sys_info.txt
fi

if test -d /etc/sysconfig/network; then
	rh=0
	decor sys_info.txt "contents of /etc/sysconfig/network"
	ls -l /etc/sysconfig/network/ >> sys_info.txt
fi

for intf in ${intrf[@]}
do
	if [ $rh == 1 ]
	then
		decor sys_info.txt "contents of /etc/sysconfig/network-scripts/ifcfg-$intf"
		cat /etc/sysconfig/network-scripts/ifcfg-$intf >> sys_info.txt 2>&1
	else
		if test -f /etc/sysconfig/network/ifcfg-$intf; then
		decor sys_info.txt "contents of contents of /etc/sysconfig/network/ifcfg-$intf"
		cat /etc/sysconfig/network/ifcfg-$intf >> sys_info.txt 2>&1
		else
			mac=`ifconfig -a | grep -i $intf | awk '{print $5}'`
			decor sys_info.txt "contents of contents of /etc/sysconfig/network/ifcfg-eth-id-$mac"
			cat /etc/sysconfig/network/ifcfg-eth-id-$mac >> sys_info.txt 2>&1
		fi
	fi
done

decor netstat_info.txt "Connection  info"
netstat -antp | grep "ESTABLISHED " | wc -l >> netstat_info.txt

for intf in ${intrf[@]}
do
		echo "Initial Counters @ `date`" >> be2net_stats_$intf.txt
		Initial=`date "+%s"` 
		ifconfig $intf >> be2net_stats_$intf.txt
		echo "" >> be2net_stats_$intf.txt
		ethtool -S $intf >> be2net_stats_$intf.txt
		echo "" >> be2net_stats_$intf.txt
		echo "/proc/interrupts for $intf" >> be2net_stats_$intf.txt
		cat /proc/interrupts | grep -i $intf >> be2net_stats_$intf.txt
done

echo "" >> netstat_info.txt
decor netstat_info.txt "netstat -s"
netstat -s >> netstat_info.txt
echo "Initial Qdisc stats @ `date`" >> qdisc_stats.txt
echo "" >> qdisc_stats.txt
decor qdisc_stats.txt "qdisc stats"
tc -s qdisc >> qdisc_stats.txt

if test -f /usr/bin/numastat; then
	echo "Get numastat information:" > numastat_info.txt
	numastat >> numastat_info.txt 2>&1
else
	echo "numastat not available" > numastat_info.txt
fi

echo "CPU usage information:" > cpu_util_info.txt
if test -f /usr/bin/mpstat; then
	 mpstat -P ALL $DELAY $COUNT >> cpu_util_info.txt 2>&1
else
	 top -bHS -d$DELAY n$COUNT >> cpu_util_info.txt 2>&1
fi

for intf in ${intrf[@]}
do
	echo "Final Counters @ `date`" >> be2net_stats_$intf.txt
	Final=`date "+%s"` 
	ifconfig $intf >> be2net_stats_$intf.txt
	echo "" >> be2net_stats_$intf.txt
	 IPaddress=$(ifconfig $intf| grep 'inet addr' |awk -F : '{print $2}'|cut -d " " -f1)
         if [[ -z "$IPaddress" ]] ;then
         	IPaddress="0.0.0.0"
         fi
         echo  "IPaddress of $eth: $IPaddress" >> be2net_stats_$intf.txt
	ethtool -S $intf >> be2net_stats_$intf.txt
	echo "/proc/interrupts for $intf" >> be2net_stats_$intf.txt
	cat /proc/interrupts | grep -i $intf >> be2net_stats_$intf.txt
        echo "IRQs of $intf: " >>be2net_stats_$intf.txt
	IRQS=$(cat /proc/interrupts | grep $intf | awk '{print $1}' | sed 's/://')
        for irq in $IRQS
        do
                echo -n "$irq: " >>be2net_stats_$intf.txt
                cat /proc/irq/$irq/smp_affinity >>be2net_stats_$intf.txt
        done

done

echo "Final netstat @ `date`" >> netstat_info.txt
echo "" >> netstat_info.txt
decor netstat_info.txt "netstat -s"
netstat -s >> netstat_info.txt
echo "Final Qdisc stats @ `date`" >> qdisc_stats.txt
echo "" >> qdisc_stats.txt
decor qdisc_stats.txt "qdisc stats"
tc -s qdisc >> qdisc_stats.txt

if test -f /usr/bin/numastat; then
	echo "Get numastat information:" >> numastat_info.txt
	numastat >> numastat_info.txt 2>&1
else
	echo "numastat not available" >> numastat_info.txt
fi

###############Data accumulated. Parse it now.###########

runtime=$(($Final - $Initial))
# Array of Rx stats
rx_stats_array=(rx_bytes rx_pkts rx_compl rx_mcast_pkts rx_post_fail rx_drops_no_skbs rx_drops_no_frags) 

#Array of Tx stats
tx_stats_array=(tx_bytes tx_pkts tx_reqs tx_wrbs tx_stops tx_compl)

#Array of be2net stats
be2net_stats_array=( rx_crc_errors rx_alignment_symbol_errors rx_pause_frames rx_control_frames rx_in_range_errors rx_out_range_errors rx_frame_too_long rx_dropped_too_small rx_dropped_too_short rx_dropped_header_too_small rx_dropped_tcp_length rx_dropped_runt rxpp_fifo_overflow_drop rx_input_fifo_overflow_drop rx_ip_checksum_errs rx_tcp_checksum_errs rx_udp_checksum_errs rx_switched_unicast_packets rx_switched_multicast_packets rx_switched_broadcast_packets rx_priority_pause_frames rx_drops_no_pbuf rx_drops_no_erx_descr rx_drops_no_tpre_descr rx_drops_too_many_frags rx_drops_mtu rxpp_fifo_overflow_drop pmem_fifo_overflow_drop jabber_events forwarded_packets eth_red_drops tx_pauseframes tx_controlframes )


for intf in ${intrf[@]}
do
	decor be2net_stats_report.txt "Stats for $intf"
	file_name=be2net_stats_$intf.txt
	#Get number of Rx rings.
	nr=`cat $file_name | grep rx_compl | wc -l`
	num_rx_rings=`expr $nr / 2`
	#Number of Tx rings in use
	nr=`cat $file_name | grep tx_compl | wc -l`
	num_tx_rings=`expr $nr / 2`
	#Number of Tx Event Queues.
	nr=`cat $file_name | grep tx_events | wc -l`
	num_tx_event_rings=`expr $nr / 2`
	echo "Number Rx rings $num_rx_rings" >> be2net_stats_report.txt
	echo "Number Tx rings $num_tx_rings" >> be2net_stats_report.txt
	echo "Number Tx Event rings $num_tx_event_rings" >> be2net_stats_report.txt
	echo "" >> be2net_stats_report.txt
	echo "----------------be2net stats----------------" >> be2net_stats_report.txt
	len=${#be2net_stats_array[*]}
	j=0
	errcnt=0
	drpcnt=0
	while [ $j -lt $len ]; do
		i=${be2net_stats_array[$j]}
		bn=`cat $file_name | grep $i | sed 's/^.*\s//' | xargs | tr ' ' - | bc`
		let "bn=0 - $bn"
		echo `echo $i | sed 's/_/ /g' | sed 's/^./\u&/'` : $bn >> be2net_stats_report.txt
		if [[ $i == *error* ]] ; then
			let "$errcnt + bn"
		fi
		if [[ $i == rx*drop* ]] || [[ $i == eth* ]] ; then
			if [[ $i != "rx_drops_no_pbuf" ]] ; then
				let "$drpcnt + $bn"
			fi
		fi
		let j++
	done
	echo "`echo Total_Rx_drop_Errors |sed 's/_/ /g'` : $errcnt" >> be2net_stats_report.txt
	echo "`echo Total_Rx_drop_Others |sed 's/_/ /g'` : $drpcnt" >> be2net_stats_report.txt
	echo "" >> be2net_stats_report.txt
	echo "----------------be2net Rx stats----------------" >> be2net_stats_report.txt
	len=${#rx_stats_array[*]}
	j=0
	while [ $j -lt $len ]; do
		i=${rx_stats_array[$j]}
		cnt=0
		for ((rx=0; rx<$num_rx_rings; rx++))
		do
			bn=`cat $file_name | grep $i | grep rxq$rx | sed 's/^.*\s//' | xargs | tr ' ' - | bc`
			let "bn=0 - $bn"
			let "cnt=$cnt + $bn"
			echo "Rx Ring $rx" `echo $i | sed 's/_/ /g'` : $bn >> be2net_stats_report.txt
		done
		echo "Total `echo $i | sed 's/_/ /g'` : $cnt" >> be2net_stats_report.txt
		# Calculating total Rx_rate in in Mbps
		if [[ "$i" == "rx_bytes" ]] ; then
			if [[ $cnt -eq 0 ]] ; then
				Rx_Rate=0
			else
				Rx_Rate=$((($cnt * 8) / 1000000))
				rxbytes=$cnt
			fi
		echo "`echo Rx_Rate |sed 's/_/ /g'` : $Rx_Rate" >> be2net_stats_report.txt
		fi
		if [[ "$i" == "rx_pkts" ]] ; then
			rxpkts=$cnt
		fi
		let j++
	done
		if [[ $rxbytes -eq 0 ]] || [[ $rxpkts -eq 0 ]] ; then
			Avg_Rx_Pkt_Size=0
		else
			Avg_Rx_Pkt_Size=$(($rxbytes / $rxpkts))
		fi
		echo "`echo Avg_Rx_Pkt_Size | sed 's/_/ /g'` : $Avg_Rx_Pkt_Size" >> be2net_stats_report.txt
		echo "" >> be2net_stats_report.txt
	echo "----------------be2net Tx stats----------------" >> be2net_stats_report.txt
	len=${#tx_stats_array[*]}
	j=0
	while [ $j -lt $len ]; do
		i=${tx_stats_array[$j]}
		cnt=0
		for ((tx=0; tx<$num_tx_rings; tx++))
		do
			bn=`cat $file_name | grep $i | grep txq$tx | sed 's/^.*\s//' | xargs | tr ' ' - | bc`
			let "bn=0 - $bn"
			let "cnt=$cnt + $bn"
			echo "Tx Ring $tx" `echo $i | sed 's/_/ /g'` : $bn >> be2net_stats_report.txt
		done
		echo "Total `echo $i | sed 's/_/ /g'` : $cnt" >> be2net_stats_report.txt
		# Calculating total Tx_rate in in Mbps
		if [[ "$i" == "tx_bytes" ]] ; then
			if [[ $cnt -eq 0 ]] ; then
				Tx_Rate=0
			else
				Tx_Rate=$((($cnt * 8) / 1000000))
				txbytes=$cnt
			fi
		echo "`echo Tx_Rate |sed 's/_/ /g'` : $Tx_Rate" >> be2net_stats_report.txt
		fi
		if [[ "$i" == "tx_pkts" ]] ; then
			txpkts=$cnt
		fi
		if [[ "$i" == "tx_reqs" ]] ; then
			txreqs=$cnt
		fi
		let j++
	done
		# Calculating  Average TX pkt size
		if [[ $txbytes -eq 0 ]] || [[ $txpkts -eq 0 ]] ; then
			Avg_Tx_Pkt_Size=0
		else
			Avg_Tx_Pkt_Size=$(($txbytes / $txpkts))
		fi
		echo "`echo Avg_Tx_Pkt_Size | sed 's/_/ /g'` : $Avg_Tx_Pkt_Size" >> be2net_stats_report.txt
		# Calculating  Average TX  request size
		if [[ $txreqs -eq 0 ]] || [[ $txbytes -eq 0 ]] ; then
			Avg_Tx_Req_Size=0
		else
			Avg_Tx_Req_Size=$(($txbytes / $txreqs))
		fi
		echo "`echo Avg_Tx_Req_Size | sed 's/_/ /g'` : $Avg_Tx_Req_Size" >> be2net_stats_report.txt
done
echo "" >> be2net_stats_report.txt
echo "----------------netstat----------------" >> be2net_stats_report.txt
retr=`cat netstat_info.txt | grep "retransmited" | awk '{print $1}' | xargs | tr ' ' - | bc`
let "retr=0 - $retr"
echo "Packets retransmitted during sampling duration : $retr" >> be2net_stats_report.txt
echo "Total Runtime : $runtime" >>be2net_stats_report.txt

# Move the generated files into a directory, tar it and cleanup.
host_name=`hostname`
rm -rf $host_name &> /dev/null
mkdir $host_name  
mv sys_lspci_info.txt $host_name/
mv sys_dmesg.txt $host_name/
mv cpu_util_info.txt $host_name/
mv sys_info.txt $host_name/
mv numastat_info.txt $host_name/
mv netstat_info.txt $host_name/
mv qdisc_stats.txt $host_name/
mv be2net_info.txt $host_name/
mv be2net_stats_report.txt $host_name/
mv be2net_stats_*.txt $host_name/

tar -cvzf $host_name.tar.gz $host_name
rm -rf $host_name

echo `date` ": $host_name.tar.gz created. Gathering Data done"
